@@ -22,7 +22,7 @@ enum transfer_type_t {
 	TYPE_STORE
 };
 
-#if (__i386__)
+#if (CPU_i386)
 
 /* instruction jump table */
 //i386op_func *cpufunctbl[256];
@@ -453,7 +453,7 @@ buserr:
 
 void install_sigsegv() {
 	sigemptyset(&sigsegv_sa.sa_mask);
-	sigsegv_sa.sa_handler = segfault_vec;
+	sigsegv_sa.sa_handler = (sighandler_t) segfault_vec;
 	sigsegv_sa.sa_flags = SA_SIGINFO;
 	sigaction(SIGSEGV, &sigsegv_sa, NULL);
 //	signal(SIGSEGV, (sighandler_t)segfault_vec);
