@@ -3,11 +3,9 @@
 #include "file.h"
 #include "parameters.h"			// load/saveSettings()
 
-#define Screen_Save()		{ hostScreen.lock(); hostScreen.saveBackground(); hostScreen.unlock(); }
+#define Screen_Save()		{ SCRLOCK; hostScreen.saveBackground(); SCRUNLOCK; }
 #define Screen_SetFullUpdate()
-#define Screen_Draw()		{ hostScreen.lock(); hostScreen.restoreBackground(); hostScreen.unlock(); }
-
-extern int Dialog_AlertDlg(const char *);
+#define Screen_Draw()		{ SCRLOCK; hostScreen.restoreBackground(); SCRUNLOCK; }
 
 /* The main dialog: */
 enum MAINDLG {
@@ -18,12 +16,14 @@ enum MAINDLG {
 	DISCS,
 	HOTKEYS,
 	KEYBOARD,
-	TOS,
+	OS,
 	VIDEO,
+/*
 	MEMORY,
 	HOSTFS,
 	CDROM,
 	INOUT,
+*/
 	text_conffile,
 	LOAD,
 	SAVE,
@@ -41,16 +41,18 @@ SGOBJ maindlg[] = {
 	{SGBOX, SG_BACKGROUND, 0, 0, 0, 40, 25, NULL},
 	{SGBOX, 0, 0, 1, 2, 38, 15, NULL},
 	{SGTEXT, 0, 0, 14, 1, 14, 1, " ARAnyM SETUP "},
-	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 3, 3, 15, 1, "About"},
-	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 3, 5, 15, 1, "Disks"},
-	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 3, 7, 15, 1, "Hotkeys"},
-	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 3, 9, 15, 1, "Keyboard&mouse"},
-	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 3, 11, 15, 1, "TOS"},
-	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 22, 3, 15, 1, "Video"},
-	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 22, 5, 15, 1, "JIT CPU"},
-	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 22, 7, 15, 1, "Host FS"},
-	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 22, 9, 15, 1, "Host CD/DVD"},
-	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 22, 11, 15, 1, "Input/Output"},
+	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 2, 3, 17, 1, "About"},
+	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 2, 5, 17, 1, "Disks"},
+	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 2, 7, 17, 1, "Hotkeys"},
+	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 2, 9, 17, 1, "Keyboard + mouse"},
+	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 2, 11, 17, 1, "Operating System"},
+	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 21, 3, 17, 1, "Video"},
+/*
+	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 21, 5, 17, 1, "JIT CPU"},
+	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 21, 7, 17, 1, "Host FS"},
+	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 21, 9, 17, 1, "Host CD/DVD"},
+	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 21, 11, 17, 1, "Input/Output"},
+*/
 	{SGTEXT, 0, 0, 4, 15, 12, 1, "Config file:"},
 	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 17, 15, 6, 1, "Load"},
 	{SGBUTTON, SG_SELECTABLE|SG_EXIT, 0, 25, 15, 6, 1, "Save"},
@@ -65,13 +67,42 @@ SGOBJ maindlg[] = {
 	{-1, 0, 0, 0, 0, 0, 0, NULL}
 };
 
+static const char *ABOUT_TEXT =
+"               ARAnyM\n"
+"               ======\n"
+"\n"
+"ARAnyM as an Open Source project has\n"
+"been developed by a loosely knit team\n"
+"of hackers accross the Net. Please see\n"
+"the doc file AUTHORS for more info.\n"
+"\n"
+"This program is free software; you can\n"
+"redistribute it and/or modify it under\n"
+"the terms of the GNU General Public\n"
+"License as published by the Free Soft-\n"
+"ware Foundation; either version 2 of\n"
+"the License, or (at your option) any\n"
+"later version.\n"
+"\n"
+"This program is distributed in the\n"
+"hope that it will be useful, but\n"
+"WITHOUT ANY WARRANTY. See the GNU Ge-\n"
+"neral Public License for more details.";
+
+static const char *HELP_TEXT = 
+"          ARAnyM SETUP menu:\n"
+"          ------------------\n"
+"Changes must be confirmed with APPLY first. Note that all changes are applied immediately to a running ARAnyM.\n"
+"Some changes require system reboot in order to take effect. It's actually safest to always reboot after any change.";
+
 static bool bReboot;
 static bool bShutdown;
 
-extern void Dialog_AboutDlg();
 extern void Dialog_DiscDlg();
+extern void Dialog_HotkeysDlg();
 extern void Dialog_KeyboardDlg();
-extern void Dialog_HelpDlg();
+extern void Dialog_OsDlg();
+extern void Dialog_VideoDlg();
 
 static char path[MAX_FILENAME_LENGTH] = "";
 
@@ -105,16 +136,16 @@ void Dialog_MainDlg()
 
 	Screen_Save();
 
-	hostScreen.lock();
+	SCRLOCK;
 	SDL_ShowCursor(SDL_ENABLE);
-	hostScreen.unlock();
+	SCRUNLOCK;
 
 	bool closeDialog = false;
 	do {
 		retbut = SDLGui_DoDialog(maindlg);
 		switch (retbut) {
 		case ABOUT:
-			Dialog_AboutDlg();
+			SDLGui_Alert(ABOUT_TEXT, ALERT_OK);
 			break;
 
 		case DISCS:
@@ -126,15 +157,22 @@ void Dialog_MainDlg()
 			break;
 
 		case HOTKEYS:
-		case TOS:
+			Dialog_HotkeysDlg();
+			break;
+		case OS:
+			Dialog_OsDlg();
+			break;
 		case VIDEO:
+			Dialog_VideoDlg();
+			break;
+/*
 		case MEMORY:
 		case HOSTFS:
 		case CDROM:
 		case INOUT:
-			Dialog_AlertDlg("Unimplemented yet.");
+			SDLGui_Alert("Unimplemented yet.", ALERT_OK);
 			break;
-
+*/
 		case LOAD:
 			LoadSettings();
 			break;
@@ -166,7 +204,7 @@ void Dialog_MainDlg()
 			break;
 
 		case HELP:
-			Dialog_HelpDlg();
+			SDLGui_Alert(HELP_TEXT, ALERT_OK);
 			break;
 
 		case CLOSE:
@@ -178,9 +216,9 @@ void Dialog_MainDlg()
 	}
 	while (!closeDialog);
 
-	hostScreen.lock();
+	SCRLOCK;
 	SDL_ShowCursor(SDL_DISABLE);
-	hostScreen.unlock();
+	SCRUNLOCK;
 }
 
 /*-----------------------------------------------------------------------*/
@@ -195,3 +233,7 @@ int GUImainDlg()
 
 	return 0;
 }
+
+/*
+vim:ts=4:sw=4:
+*/
