@@ -32,6 +32,10 @@
 
 #include "parameters.h"
 
+#ifdef ENABLE_OPENGL
+#include <SDL_opengl.h>
+#endif
+
 /**
  * This macro handles the endianity for 24 bit per item data
  **/
@@ -74,13 +78,15 @@ class HostScreen {
 	int selectVideoMode(SDL_Rect **modes, uint32 *width, uint32 *height);
 	void searchVideoMode( uint32 *width, uint32 *height, uint32 *bpp );
 
+#ifdef ENABLE_OPENGL
 	// OpenGL stuff
 	SDL_Surface *SdlGlSurface;
-	unsigned int SdlGlTexObj;
+	GLuint SdlGlTexObj;
 	unsigned int SdlGlTextureWidth;
 	unsigned int SdlGlTextureHeight;
 	uint8 *SdlGlTexture;
 	int SdlGlWidth, SdlGlHeight;
+#endif
 	SDL_bool npot_texture;		/* Use non power of two texture ? */
 	SDL_bool rect_texture;		/* Use rectangle texture ? */
 	int rect_target;
@@ -184,6 +190,9 @@ class HostScreen {
 
 	// Toggle Window/FullScreen mode
 	void   toggleFullScreen();
+
+	int	refreshCounter;
+	void	refresh(void);
 };
 
 
@@ -370,58 +379,152 @@ inline void HostScreen::blitArea( int16 sx, int16 sy, int16 dx, int16 dy, int16
  */
 inline void HostScreen::bitplaneToChunky( uint16 *atariBitplaneData, uint16 bpp, uint8 colorValues[16] )
 {
-	memset( colorValues, 0, 16 ); // clear the color values for the 16 pixels (word length)
+	uint32 a, b, c, d, x;
 
-	for (int l = bpp - 1; l >= 0; l--) {
-		uint16 data = atariBitplaneData[l]; // note: this is about 2000 dryhstones sppedup (the local variable)
-
-#if SDL_BYTEORDER != SDL_BIG_ENDIAN
-		colorValues[ 0] <<= 1;	colorValues[ 0] |= (data >>  7) & 1;
-		colorValues[ 1] <<= 1;	colorValues[ 1] |= (data >>  6) & 1;
-		colorValues[ 2] <<= 1;	colorValues[ 2] |= (data >>  5) & 1;
-		colorValues[ 3] <<= 1;	colorValues[ 3] |= (data >>  4) & 1;
-		colorValues[ 4] <<= 1;	colorValues[ 4] |= (data >>  3) & 1;
-		colorValues[ 5] <<= 1;	colorValues[ 5] |= (data >>  2) & 1;
-		colorValues[ 6] <<= 1;	colorValues[ 6] |= (data >>  1) & 1;
-		colorValues[ 7] <<= 1;	colorValues[ 7] |= (data >>  0) & 1;
-
-		colorValues[ 8] <<= 1;	colorValues[ 8] |= (data >> 15) & 1;
-		colorValues[ 9] <<= 1;	colorValues[ 9] |= (data >> 14) & 1;
-		colorValues[10] <<= 1;	colorValues[10] |= (data >> 13) & 1;
-		colorValues[11] <<= 1;	colorValues[11] |= (data >> 12) & 1;
-		colorValues[12] <<= 1;	colorValues[12] |= (data >> 11) & 1;
-		colorValues[13] <<= 1;	colorValues[13] |= (data >> 10) & 1;
-		colorValues[14] <<= 1;	colorValues[14] |= (data >>  9) & 1;
-		colorValues[15] <<= 1;	colorValues[15] |= (data >>  8) & 1;
+	/* Obviously the different cases can be broken out in various
+	 * ways to lessen the amount of work needed for <8 bit modes.
+	 * It's doubtful if the usage of those modes warrants it, though.
+	 * The branches below should be ~100% correctly predicted and
+	 * thus be more or less for free.
+	 * Getting the palette values inline does not seem to help
+	 * enough to worry about. The palette lookup is much slower than
+	 * this code, though, so it would be nice to do something about it.
+	 */
+	if (bpp >= 4) {
+		d = *(uint32 *)&atariBitplaneData[0];
+		c = *(uint32 *)&atariBitplaneData[2];
+		if (bpp == 4) {
+			a = b = 0;
+		} else {
+			b = *(uint32 *)&atariBitplaneData[4];
+			a = *(uint32 *)&atariBitplaneData[6];
+		}
+	} else {
+		a = b = c = 0;
+		if (bpp == 2) {
+			d = *(uint32 *)&atariBitplaneData[0];
+		} else {
+#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+			d = atariBitplaneData[0]<<16;
 #else
-		colorValues[ 0] <<= 1;	colorValues[ 0] |= (data >> 15) & 1;
-		colorValues[ 1] <<= 1;	colorValues[ 1] |= (data >> 14) & 1;
-		colorValues[ 2] <<= 1;	colorValues[ 2] |= (data >> 13) & 1;
-		colorValues[ 3] <<= 1;	colorValues[ 3] |= (data >> 12) & 1;
-		colorValues[ 4] <<= 1;	colorValues[ 4] |= (data >> 11) & 1;
-		colorValues[ 5] <<= 1;	colorValues[ 5] |= (data >> 10) & 1;
-		colorValues[ 6] <<= 1;	colorValues[ 6] |= (data >>  9) & 1;
-		colorValues[ 7] <<= 1;	colorValues[ 7] |= (data >>  8) & 1;
-
-		colorValues[ 8] <<= 1;	colorValues[ 8] |= (data >>  7) & 1;
-		colorValues[ 9] <<= 1;	colorValues[ 9] |= (data >>  6) & 1;
-		colorValues[10] <<= 1;	colorValues[10] |= (data >>  5) & 1;
-		colorValues[11] <<= 1;	colorValues[11] |= (data >>  4) & 1;
-		colorValues[12] <<= 1;	colorValues[12] |= (data >>  3) & 1;
-		colorValues[13] <<= 1;	colorValues[13] |= (data >>  2) & 1;
-		colorValues[14] <<= 1;	colorValues[14] |= (data >>  1) & 1;
-		colorValues[15] <<= 1;	colorValues[15] |= (data >>  0) & 1;
+			d = atariBitplaneData[0];
 #endif
+		}
 	}
+
+	x = a;
+	a =  (a & 0xf0f0f0f0)       | ((c & 0xf0f0f0f0) >> 4);
+	c = ((x & 0x0f0f0f0f) << 4) |  (c & 0x0f0f0f0f);
+	x = b;
+	b =  (b & 0xf0f0f0f0)       | ((d & 0xf0f0f0f0) >> 4);
+	d = ((x & 0x0f0f0f0f) << 4) |  (d & 0x0f0f0f0f);
+
+	x = a;
+	a =  (a & 0xcccccccc)       | ((b & 0xcccccccc) >> 2);
+	b = ((x & 0x33333333) << 2) |  (b & 0x33333333);
+	x = c;
+	c =  (c & 0xcccccccc)       | ((d & 0xcccccccc) >> 2);
+	d = ((x & 0x33333333) << 2) |  (d & 0x33333333);
+
+#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+	a = (a & 0x5555aaaa) | ((a & 0x00005555) << 17) | ((a & 0xaaaa0000) >> 17);
+	b = (b & 0x5555aaaa) | ((b & 0x00005555) << 17) | ((b & 0xaaaa0000) >> 17);
+	c = (c & 0x5555aaaa) | ((c & 0x00005555) << 17) | ((c & 0xaaaa0000) >> 17);
+	d = (d & 0x5555aaaa) | ((d & 0x00005555) << 17) | ((d & 0xaaaa0000) >> 17);
+	
+	colorValues[ 8] = a;
+	a >>= 8;
+	colorValues[ 0] = a;
+	a >>= 8;
+	colorValues[ 9] = a;
+	a >>= 8;
+	colorValues[ 1] = a;
+	
+	colorValues[10] = b;
+	b >>= 8;
+	colorValues[ 2] = b;
+	b >>= 8;
+	colorValues[11] = b;
+	b >>= 8;
+	colorValues[ 3] = b;
+	
+	colorValues[12] = c;
+	c >>= 8;
+	colorValues[ 4] = c;
+	c >>= 8;
+	colorValues[13] = c;
+	c >>= 8;
+	colorValues[ 5] = c;
+	
+	colorValues[14] = d;
+	d >>= 8;
+	colorValues[ 6] = d;
+	d >>= 8;
+	colorValues[15] = d;
+	d >>= 8;
+	colorValues[ 7] = d;
+#else
+	a = (a & 0xaaaa5555) | ((a & 0x0000aaaa) << 15) | ((a & 0x55550000) >> 15);
+	b = (b & 0xaaaa5555) | ((b & 0x0000aaaa) << 15) | ((b & 0x55550000) >> 15);
+	c = (c & 0xaaaa5555) | ((c & 0x0000aaaa) << 15) | ((c & 0x55550000) >> 15);
+	d = (d & 0xaaaa5555) | ((d & 0x0000aaaa) << 15) | ((d & 0x55550000) >> 15);
+
+	colorValues[ 1] = a;
+	a >>= 8;
+	colorValues[ 9] = a;
+	a >>= 8;
+	colorValues[ 0] = a;
+	a >>= 8;
+	colorValues[ 8] = a;
+
+	colorValues[ 3] = b;
+	b >>= 8;
+	colorValues[11] = b;
+	b >>= 8;
+	colorValues[ 2] = b;
+	b >>= 8;
+	colorValues[10] = b;
+
+	colorValues[ 5] = c;
+	c >>= 8;
+	colorValues[13] = c;
+	c >>= 8;
+	colorValues[ 4] = c;
+	c >>= 8;
+	colorValues[12] = c;
+
+	colorValues[ 7] = d;
+	d >>= 8;
+	colorValues[15] = d;
+	d >>= 8;
+	colorValues[ 6] = d;
+	d >>= 8;
+	colorValues[14] = d;
+#endif
 }
 
-
-
 #endif
 
-
 /*
  * $Log: hostscreen.h,v $
+ * Revision 1.70  2006-12-17 14:46:00  philipp
+ * Fix: ENABLE_OPENGL is only defined after "parameters.h" has been included.
+ *
+ * Revision 1.69  2006/12/13 00:11:05  philipp
+ * Small fixes for OpenGLVdiDriver bugs which bothered MacAranym (compilation and runtime errors/crashes).
+ *
+ * Revision 1.68  2006/11/10 23:21:26  pmandin
+ * Move video refresh from main to hostscreen. More to follow soon.
+ *
+ * Revision 1.67  2006/08/25 18:34:22  philipp
+ * Fixed p2c routine for big endian, colors are now correct.
+ *
+ * Revision 1.66  2006/08/25 13:16:10  pmandin
+ * big endian patch from philipp donze
+ *
+ * Revision 1.65  2006/05/31 22:06:18  johan
+ * "Real" planar to chunky code for little endian CPUs.
+ *
  * Revision 1.64  2005/06/14 16:45:04  pmandin
  * Use rectangle texture when available
  *
