@@ -1,7 +1,7 @@
 /*
  * input.cpp - handling of keyboard/mouse input
  *
- * Copyright (c) 2001-2005 Petr Stehlik of ARAnyM dev team (see AUTHORS)
+ * Copyright (c) 2001-2006 Petr Stehlik of ARAnyM dev team (see AUTHORS)
  *
  * This file is part of the ARAnyM project which builds a new and powerful
  * TOS/FreeMiNT compatible virtual machine running on almost any hardware.
@@ -32,8 +32,10 @@
 #  include "sdlgui.h"
 #endif
 
-#include "nf_objs.h"
-#include "nfvdi.h"
+#ifdef NFVDI_SUPPORT
+# include "nf_objs.h"
+# include "nfvdi.h"
+#endif
 
 #define DEBUG 0
 #include "debug.h"
@@ -103,6 +105,11 @@ void InputReset()
 	capslockState = (SDL_GetModState() & KMOD_CAPS) != 0;
 }
 
+void InputExit()
+{
+	grabMouse(false);	// release mouse
+	hideMouse(false);	// show it
+}
 
 bool grabMouse(bool grab)
 {
@@ -186,7 +193,7 @@ static int keysymToAtari(SDL_keysym keysym)
 	switch (keysym.sym) {
 	  case SDLK_LMETA:
 	  case SDLK_RMETA:
-	    return 0x1D;
+	    return 0;
 		break;
 	  case SDLK_MODE: /* passthru */ /* Alt Gr key according to SDL docs */
 	  case SDLK_RALT:
@@ -202,7 +209,7 @@ static int keysymToAtari(SDL_keysym keysym)
 		break;
 	  case SDLK_RCTRL:
 	  case SDLK_LCTRL:
-	    return 0;
+	    return 0x1D;
 	  default:
 	    break;
 	}
@@ -420,6 +427,8 @@ static const int GUI_RETURN_INFO = (SDL_USEREVENT+1);
 // running in a different thread
 static int open_gui(void * /*ptr*/)
 {
+	bool fullscreen = bx_options.video.fullscreen;
+
 	hostScreen.openGUI();
 	int status = GUImainDlg();
 
@@ -431,6 +440,16 @@ static int open_gui(void * /*ptr*/)
 	SDL_PeepEvents(&ev, 1, SDL_ADDEVENT, SDL_EVENTMASK(GUI_RETURN_INFO));
 
 	hostScreen.closeGUI();
+
+	// small hack to toggle fullscreen from the SETUP GUI
+	if (bx_options.video.fullscreen != fullscreen) {
+		bx_options.video.fullscreen = fullscreen;
+
+		hostScreen.toggleFullScreen();
+		if (bx_options.video.fullscreen && !grabbedMouse)
+			grabTheMouse();
+	}
+
 	return 0;
 }
 
@@ -456,7 +475,7 @@ void kill_GUI_thread()
 #endif /* SDL_GUI */
 
 #define CHECK_HOTKEY(Hotkey) ((bx_options.hotkeys.Hotkey.sym == 0 || sym == bx_options.hotkeys.Hotkey.sym) && masked_mod == bx_options.hotkeys.Hotkey.mod)
-static void process_keyboard_event(SDL_Event &event)
+static void process_keyboard_event(const SDL_Event &event)
 {
 	SDL_keysym keysym = event.key.keysym;
 	SDLKey sym = keysym.sym;
@@ -475,12 +494,15 @@ static void process_keyboard_event(SDL_Event &event)
 #endif /* SDL_GUI */
 
 #if FLIGHT_RECORDER
-	static bool flight_was_active = false;
-	bool flight_should_be_active = (state & (KMOD_SHIFT)) == KMOD_SHIFT;
-	if (flight_was_active != flight_should_be_active) {
-		cpu_flight_recorder(flight_should_be_active);
-		flight_was_active = flight_should_be_active;
-		panicbug("Flight was %sactivated!", (flight_should_be_active ? "" : "DE"));
+	static bool flight_is_active = false;
+	bool flight_turn_on = (state & (KMOD_SHIFT)) == KMOD_RSHIFT;
+	bool flight_turn_off = (state & (KMOD_SHIFT)) == KMOD_LSHIFT;
+	bool flight_changed = (flight_turn_on && !flight_is_active) ||
+						  (flight_turn_off && flight_is_active);
+	if (flight_changed) {
+		flight_is_active = ! flight_is_active;
+		cpu_flight_recorder(flight_is_active);
+		panicbug("Flight was %sactivated!", (flight_is_active ? "" : "DE"));
 	}
 #endif
 
@@ -515,18 +537,28 @@ static void process_keyboard_event(SDL_Event &event)
 		}
 #ifdef SDL_GUI
 		else if (CHECK_HOTKEY(setup)) {
+			/* release shifters (if any) */
+			if ( bx_options.hotkeys.setup.mod & KMOD_LSHIFT )
+				getIKBD()->SendKey(0x80 | 0x2a);
+			if ( bx_options.hotkeys.setup.mod & KMOD_RSHIFT )
+				getIKBD()->SendKey(0x80 | 0x36);
+			if ( bx_options.hotkeys.setup.mod & KMOD_CTRL )
+				getIKBD()->SendKey(0x80 | 0x1d);
+			if ( bx_options.hotkeys.setup.mod & KMOD_LALT )
+				getIKBD()->SendKey(0x80 | 0x38);
+			if ( bx_options.hotkeys.setup.mod & KMOD_MODE|KMOD_RALT )
+				getIKBD()->SendKey(0x80 | RALT_ATARI_SCANCODE);
+			
 			start_GUI_thread();
 			send2Atari = false;
 		}
 #endif
 #ifdef DEBUGGER
 		else if (CHECK_HOTKEY(debug)) {
-			if (bx_options.startup.debugger) {
-				releaseTheMouse();
-				canGrabMouseAgain = false;	// let it leave our window
-				// activate debugger
-				activate_debugger();
-			}
+			releaseTheMouse();
+			canGrabMouseAgain = false;	// let it leave our window
+			// activate debugger
+			activate_debugger();
 			send2Atari = false;
 		}
 #endif
@@ -590,21 +622,17 @@ static void process_keyboard_event(SDL_Event &event)
 	}
 }
 
-static void process_mouse_event(SDL_Event event)
+static void process_mouse_event(const SDL_Event &event)
 {
+	bool mouse_exit = false;
+#ifdef NFVDI_SUPPORT
 	static NF_Base* fvdi = NULL;
 	bool fvdi_events = false;
-	bool mouse_exit = false;
 
 	if (!fvdi) {
-		unsigned int i;
-		for(i = 0; i < nf_objs_cnt; i++) {
-			if (strcasecmp("fVDI", nf_objects[i]->name()) == 0) {
-			    fvdi = nf_objects[i];
-			    break;
-			}
-		}
+		fvdi = NFGetDriver("fVDI");
 	}
+#endif
 
 #ifdef SDL_GUI
 	if (hostScreen.isGUIopen()) {
@@ -637,8 +665,10 @@ static void process_mouse_event(SDL_Event event)
 				grabTheMouse();
 		}
 		else if (event.button.button == 4) {	/* mouse wheel Up */
+#ifdef NFVDI_SUPPORT
 			if (fvdi != NULL && fvdi->dispatch(0xc00100ff) == 0)
 				return;
+#endif
 
 			if (bx_options.ikbd.wheel_eiffel) {
 				getIKBD()->SendKey(0xF6);
@@ -656,8 +686,10 @@ static void process_mouse_event(SDL_Event event)
 			return;
 		}
 		else if (event.button.button == 5) {	/* mouse wheel Down */
+#ifdef NFVDI_SUPPORT
 			if (fvdi != NULL && fvdi->dispatch(0xc0010001) == 0)
 				return;
+#endif
 
 			if (bx_options.ikbd.wheel_eiffel) {
 				getIKBD()->SendKey(0xF6);
@@ -696,10 +728,12 @@ static void process_mouse_event(SDL_Event event)
 		xrel = eve.xrel;
 		yrel = eve.yrel;
 
+#ifdef NFVDI_SUPPORT
 		if (fvdi != NULL) {
 			if (fvdi->dispatch(0x80008000 | (eve.x << 16) | (eve.y)) == 0)
 				fvdi_events = true;
 		}
+#endif
 		// Can't use the method below to get out of the window
 		// if the events are reported directly, since it only
 		// works with hidden mouse pointer.
@@ -707,6 +741,7 @@ static void process_mouse_event(SDL_Event event)
 		mouse_exit = (eve.x == 0) && (eve.y == 0);
 	}
 
+#ifdef NFVDI_SUPPORT
 	if (lastbut != but) {
 		if (fvdi != NULL && (fvdi->dispatch(0xb0000000 | but) == 0))
 			fvdi_events = true;
@@ -717,6 +752,7 @@ static void process_mouse_event(SDL_Event event)
 			mouseOut = true;
 		return;
 	}
+#endif
 
 	// send the mouse data packet
 	if (xrel || yrel || lastbut != but) {
@@ -740,7 +776,7 @@ static void process_mouse_event(SDL_Event event)
 	}
 }
 
-static void process_active_event(SDL_Event event)
+static void process_active_event(const SDL_Event &event)
 {
 	// if we have input focus
 	if (SDL_GetAppState() & SDL_APPINPUTFOCUS) {
@@ -783,7 +819,7 @@ static void process_active_event(SDL_Event event)
 
 SDL_Joystick *sdl_joystick;
 
-static void process_joystick_event(SDL_Event event)
+static void process_joystick_event(const SDL_Event &event)
 {
 	switch(event.type) {
 		case SDL_JOYAXISMOTION:
