@@ -29,10 +29,12 @@
 #include "hostfs.h"
 #include "ethernet.h"
 #include "debugprintf.h"
-#include "nfvdi.h"
-#include "nfvdi_soft.h"
-#ifdef ENABLE_OPENGL
-# include "nfvdi_opengl.h"
+#ifdef NFVDI_SUPPORT
+# include "nfvdi.h"
+# include "nfvdi_soft.h"
+# ifdef ENABLE_OPENGL
+#  include "nfvdi_opengl.h"
+# endif
 #endif
 #ifdef NFCDROM_SUPPORT
 # include "nfcdrom.h"
@@ -52,6 +54,10 @@
 #ifdef NFJPEG_SUPPORT
 # include "nfjpeg.h"
 #endif
+#ifdef NFCLIPBRD_SUPPORT
+# include "nfclipbrd.h"
+#endif
+
 /* add your NatFeat class definition here */
 
 /*--- Defines ---*/
@@ -86,12 +92,14 @@ void NFCreate(void)
 	NFAdd(new XHDIDriver);
 	NFAdd(new AUDIODriver);
 
+#ifdef NFVDI_SUPPORT
 # ifdef ENABLE_OPENGL
 	if ((strcmp("opengl", bx_options.natfeats.vdi_driver)==0) && bx_options.opengl.enabled)
 		NFAdd(new OpenGLVdiDriver);
 	else 
 # endif
 		NFAdd(new SoftVdiDriver);
+#endif
 
 #ifdef HOSTFS_SUPPORT
 	NFAdd(new HostFs);
@@ -126,6 +134,9 @@ void NFCreate(void)
 	NFAdd(new JpegDriver);
 #endif
 
+#ifdef NFCLIPBRD_SUPPORT
+	NFAdd(new ClipbrdNatFeat);
+#endif
 	/* add your NatFeat object declaration here */
 }
 
@@ -158,3 +169,13 @@ void NFReset(void)
 		}
 	}
 }
+
+NF_Base *NFGetDriver(char *name)
+{
+	for (unsigned int i=0; i<nf_objs_cnt; i++) {
+		if (strcasecmp(name, nf_objects[i]->name())==0) {
+			return nf_objects[i];
+		}
+	}
+	return NULL;
+}
