@@ -18,10 +18,6 @@
 	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#include <stdlib.h>
-#include <SDL_cdrom.h>
-#include <SDL_endian.h>
-
 #include "sysdeps.h"
 #include "cpu_emulation.h"
 #include "parameters.h"
@@ -31,6 +27,10 @@
 #define DEBUG 0
 #include "debug.h"
 
+#include <cstdlib>
+#include <SDL_cdrom.h>
+#include <SDL_endian.h>
+
 /*--- Defines ---*/
 
 #define NFCD_NAME	"nf:cdrom: "
