@@ -1,7 +1,7 @@
 /*
  * parameters.cpp - parameter init/load/save code
  *
- * Copyright (c) 2001-2005 ARAnyM developer team (see AUTHORS)
+ * Copyright (c) 2001-2007 ARAnyM developer team (see AUTHORS)
  *
  * Authors:
  *  MJ		Milan Jurik
@@ -30,8 +30,8 @@
 #include "config.h"
 #include "parameters.h"
 #include "tools.h"		// for safe_strncpy()
-#include "gdbstub.h"
 #include "host.h"
+#include "host_filesys.h"
 
 #define DEBUG 0
 #include "debug.h"
@@ -85,12 +85,8 @@ static struct option const long_options[] =
 #endif
   {"floppy", required_argument, 0, 'a'},
   {"resolution", required_argument, 0, 'r'},
-#if (defined(DEBUGGER) || defined(GDBSTUB))
+#ifdef DEBUGGER
   {"debug", no_argument, 0, 'D'},
-# ifdef GDBSTUB
-  {"port", required_argument, 0, 'p'},
-  {"base", required_argument, 0, 'b'},
-# endif
 #endif
   {"fullscreen", no_argument, 0, 'f'},
   {"nomouse", no_argument, 0, 'N'},
@@ -111,6 +107,7 @@ static struct option const long_options[] =
 #ifdef ENABLE_LILO
   {"lilo", no_argument, 0, 'l'},
 #endif
+  {"display", required_argument, 0, 'P'},
   {NULL, 0, NULL, 0}
 };
 
@@ -137,6 +134,7 @@ static char config_file[512];
 # define XIF_HOST_IP	"192.168.0.1"
 # define XIF_ATARI_IP	"192.168.0.2"
 # define XIF_NETMASK	"255.255.255.0"
+# define XIF_MAC_ADDR	"00:41:45:54:48:30"		// just made up from \0AETH0
 #endif
 
 static bool saveConfigFile = false;
@@ -147,6 +145,100 @@ static bx_atadevice_options_t *diskc = &bx_options.atadevice[0][0];
 static bx_atadevice_options_t *diskd = &bx_options.atadevice[0][1];
 
 
+void expand_path(char *buf, size_t buf_size)
+{
+	char *path = buf;
+	if ( !strlen(path) )
+		return;
+
+	char prefix[2048];
+	size_t prefixLen = 0;
+
+	if ( path[0] == '~' && (path[1] == '/' || path[1] == '\\') ) {
+		// replace with the home folder path
+		Host::getHomeFolder(prefix, sizeof(prefix));
+		strcat(prefix, DIRSEPARATOR);
+		prefixLen = strlen( prefix );
+		path+=2;
+	} else if (path[0] == '*' && (path[1] == '/' || path[1] == '\\') ) {
+		Host::getDataFolder(prefix, sizeof(prefix));
+		strcat(prefix, DIRSEPARATOR);
+		prefixLen = strlen( prefix );
+		path+=2;
+	} else if ( path[0] != '/' && path[0] != '\\' && path[1] != ':' ) {
+		// path relative to config file
+		strcpy(prefix, config_file);
+		// find the last dirseparator
+		const char *slash = &prefix[strlen(prefix)];
+		while ( --slash >= prefix &&
+			*slash != '/' &&
+			*slash != '\\' );
+		// get the path part (in front of the slash)
+		prefixLen = slash - prefix + 1;
+	}
+
+	if ( prefixLen > 0 ) {
+		if ( buf_size >= prefixLen + strlen(path) ) {
+			memmove( buf + prefixLen, path, strlen(path)+1 );	
+			memmove( buf, prefix, prefixLen );
+		} else {
+			panicbug("Error - config entry size is insufficient");
+		}
+	}
+}
+
+void compress_path(char *path)
+{
+	if ( !strlen(path) )
+		return;
+
+	char prefix[2048];
+	size_t prefixLen = 0;
+	char *replacement = NULL;
+
+	// path relative to config file
+	strcpy(prefix, config_file);
+	// find the last dirseparator
+	const char *slash = &prefix[strlen(prefix)];
+	while ( --slash >= prefix &&
+		*slash != '/' &&
+		*slash != '\\' );
+	// get the path part (in front of the slash)
+	prefixLen = slash - prefix + 1;
+
+	if (prefixLen && strncmp(path, prefix, prefixLen) == 0) {
+		replacement = "";
+		D(bug("%s matches %.*s", path, prefixLen, prefix));
+	} 
+	else 
+	{
+		Host::getDataFolder(prefix, sizeof(prefix));
+		prefixLen = strlen(prefix);
+		if (prefixLen && strncmp(path, prefix, prefixLen) == 0) {
+			replacement = "*";
+			D(bug("%s matches %.*s", path, prefixLen, prefix));
+		} 
+		else 
+		{
+			/* Check if home prefix matches */
+			Host::getHomeFolder(prefix, sizeof(prefix));
+			prefixLen = strlen(prefix);
+			if (prefixLen && strncmp(path, prefix, prefixLen) == 0) {
+				replacement = "~";
+				D(bug("%s matches %.*s", path, prefixLen, prefix));
+			}
+		}
+	}
+
+	if (replacement) {
+		int len1 = strlen(replacement);
+		int len2 = strlen(path+prefixLen);
+		memmove(path+len1, path+prefixLen, len2+1);
+		memcpy(path, replacement, len1);
+	}
+}
+
+
 // configuration file 
 /*************************************************************************/
 struct Config_Tag global_conf[]={
@@ -155,8 +247,10 @@ struct Config_Tag global_conf[]={
 	{ "TOS", Path_Tag, bx_options.tos_path, sizeof(bx_options.tos_path), 0},
 	{ "EmuTOS", Path_Tag, bx_options.emutos_path, sizeof(bx_options.emutos_path), 0},
 	{ "Bootstrap", Path_Tag, bx_options.bootstrap_path, sizeof(bx_options.bootstrap_path), 0},
+	{ "BootstrapArgs", String_Tag, bx_options.bootstrap_args, sizeof(bx_options.bootstrap_args), 0},
 	{ "BootDrive", Char_Tag, &bx_options.bootdrive, 0, 0},
 	{ "AutoGrabMouse", Bool_Tag, &bx_options.autoMouseGrab, 0, 0},
+	{ "GMTime", Bool_Tag, &bx_options.gmtime, 0, 0},
 #ifdef ENABLE_EPSLIMITER
 	{ "EpsEnabled", Bool_Tag, &bx_options.cpu.eps_enabled, 0, 0},
 	{ "EpsMax", Int_Tag, &bx_options.cpu.eps_max, 0, 0},
@@ -166,7 +260,11 @@ struct Config_Tag global_conf[]={
 
 void preset_global()
 {
+  strcpy(bx_options.tos_path, TOS_FILENAME);
+  strcpy(bx_options.emutos_path, EMUTOS_FILENAME);
+  strcpy(bx_options.bootstrap_path, FREEMINT_FILENAME);
   bx_options.autoMouseGrab = true;
+  bx_options.gmtime = false;	// use localtime by default
   strcpy(bx_options.floppy.path, "");
 #ifdef FixedSizeFastRAM
   FastRAMSize = FixedSizeFastRAM * 1024 * 1024;
@@ -184,11 +282,25 @@ void postload_global()
 #ifndef FixedSizeFastRAM
 	FastRAMSize = bx_options.fastram * 1024 * 1024;
 #endif
+	if (!isalpha(bx_options.bootdrive))
+		bx_options.bootdrive = 0;
+
+	expand_path(bx_options.floppy.path, sizeof(bx_options.floppy.path));
+	expand_path(bx_options.tos_path, sizeof(bx_options.tos_path));
+	expand_path(bx_options.emutos_path, sizeof(bx_options.emutos_path));
+	expand_path(bx_options.bootstrap_path, sizeof(bx_options.bootstrap_path));
 }
 
 void presave_global()
 {
+	compress_path(bx_options.floppy.path);
+	compress_path(bx_options.tos_path);
+	compress_path(bx_options.emutos_path);
+	compress_path(bx_options.bootstrap_path);
+
 	bx_options.fastram = FastRAMSize / 1024 / 1024;
+	if (bx_options.bootdrive == 0)
+		bx_options.bootdrive = ' ';
 }
 
 /*************************************************************************/
@@ -204,11 +316,6 @@ void preset_startup()
 {
   bx_options.startup.debugger = false;
   bx_options.startup.grabMouseAllowed = true;
-#ifdef GDBSTUB
-  bx_options.gdbstub.text_base = GDBSTUB_TEXT_BASE;
-  bx_options.gdbstub.data_base = GDBSTUB_DATA_BASE;
-  bx_options.gdbstub.bss_base = GDBSTUB_BSS_BASE;
-#endif
 }
 
 void postload_startup()
@@ -282,7 +389,8 @@ void preset_video()
   bx_options.video.fullscreen = false;		// Boot in Fullscreen
   bx_options.video.boot_color_depth = -1;	// Boot in color depth
   bx_options.video.monitor = -1;			// preserve default NVRAM monitor
-  bx_options.video.refresh = 2;			// 25 Hz update
+  bx_options.video.refresh = 2;				// 25 Hz update
+  strcpy(bx_options.video.window_pos, "");	// ARAnyM window position on screen
 }
 
 void postload_video()
@@ -299,6 +407,7 @@ void presave_video()
 struct Config_Tag opengl_conf[]={
 	{ "Enabled", Bool_Tag, &bx_options.opengl.enabled, 0, 0},
 	{ "Filtered", Bool_Tag, &bx_options.opengl.filtered, 0, 0},
+	{ "Library", Path_Tag, bx_options.opengl.library, sizeof(bx_options.opengl.library), 0},
 	{ NULL , Error_Tag, NULL, 0, 0 }
 };
 
@@ -306,6 +415,7 @@ void preset_opengl()
 {
   bx_options.opengl.enabled = false;
   bx_options.opengl.filtered = false;
+  strcpy(bx_options.opengl.library, "");
 }
 
 void postload_opengl()
@@ -398,10 +508,14 @@ void preset_ide()
 
 void postload_ide()
 {
+	expand_path(diskc->path, sizeof(diskc->path));
+	expand_path(diskd->path, sizeof(diskd->path));
 }
 
 void presave_ide()
 {
+	compress_path(diskc->path);
+	compress_path(diskd->path);
 }
 
 /*************************************************************************/
@@ -445,10 +559,16 @@ void preset_disk()
 
 void postload_disk()
 {
+	for(int i=0; i<DISKS; i++) {
+		expand_path(bx_options.disks[i].path, sizeof(bx_options.disks[i].path));
+	}
 }
 
 void presave_disk()
 {
+	for(int i=0; i<DISKS; i++) {
+		compress_path(bx_options.disks[i].path);
+	}
 }
 
 /*************************************************************************/
@@ -497,6 +617,8 @@ void preset_arafs()
 void postload_arafs()
 {
 	for(int i=0; i < 'Z'-'A'+1; i++) {
+		expand_path(bx_options.aranymfs[i].configPath, sizeof(bx_options.aranymfs[i].configPath));
+		
 		safe_strncpy(bx_options.aranymfs[i].rootPath, bx_options.aranymfs[i].configPath, sizeof(bx_options.aranymfs[i].rootPath));
 		int len = strlen(bx_options.aranymfs[i].configPath);
 		bx_options.aranymfs[i].halfSensitive = true;
@@ -515,10 +637,11 @@ void presave_arafs()
 	for(int i=0; i < 'Z'-'A'+1; i++) {
 		safe_strncpy(bx_options.aranymfs[i].configPath, bx_options.aranymfs[i].rootPath, sizeof(bx_options.aranymfs[i].configPath));
 		if ( strlen(bx_options.aranymfs[i].rootPath) > 0 &&
-			 bx_options.aranymfs[i].halfSensitive ) {
+			 !bx_options.aranymfs[i].halfSensitive ) {
 			// set the halfSensitive indicator
 			strcat( bx_options.aranymfs[i].configPath, ":" );
 		}
+		compress_path(bx_options.aranymfs[i].configPath);
 	}
 }
 
@@ -530,6 +653,7 @@ void presave_arafs()
 	{ "HostIP", String_Tag, &Eth->ip_host, sizeof(Eth->ip_host), 0}, \
 	{ "AtariIP", String_Tag, &Eth->ip_atari, sizeof(Eth->ip_atari), 0}, \
 	{ "Netmask", String_Tag, &Eth->netmask, sizeof(Eth->netmask), 0}, \
+	{ "MAC", String_Tag, &Eth->mac_addr, sizeof(Eth->mac_addr), 0}, \
 	{ NULL , Error_Tag, NULL, 0, 0 } \
 }
 
@@ -552,6 +676,7 @@ void preset_ethernet()
 	safe_strncpy(ETH(0, ip_host), XIF_HOST_IP, sizeof(ETH(0, ip_host)));
 	safe_strncpy(ETH(0, ip_atari), XIF_ATARI_IP, sizeof(ETH(0, ip_atari)));
 	safe_strncpy(ETH(0, netmask), XIF_NETMASK, sizeof(ETH(0, netmask)));
+	safe_strncpy(ETH(0, mac_addr), XIF_MAC_ADDR, sizeof(ETH(0, mac_addr)));
 
 	// ETH[1] - ETH[MAX_ETH] are empty by default
 	for(int i=1; i<MAX_ETH; i++) {
@@ -560,6 +685,7 @@ void preset_ethernet()
 		*ETH(i, ip_host) = '\0';
 		*ETH(i, ip_atari) = '\0';
 		*ETH(i, netmask) = '\0';
+		*ETH(i, mac_addr) = '\0';
 	}
 }
 
@@ -575,9 +701,9 @@ void presave_ethernet()
 #define LILO(x) bx_options.lilo.x
 
 struct Config_Tag lilo_conf[]={
-	{ "Kernel", String_Tag, &LILO(kernel), sizeof(LILO(kernel)), 0},
+	{ "Kernel", Path_Tag, &LILO(kernel), sizeof(LILO(kernel)), 0},
 	{ "Args", String_Tag, &LILO(args), sizeof(LILO(args)), 0},
-	{ "Ramdisk", String_Tag, &LILO(ramdisk), sizeof(LILO(ramdisk)), 0},
+	{ "Ramdisk", Path_Tag, &LILO(ramdisk), sizeof(LILO(ramdisk)), 0},
 	{ NULL , Error_Tag, NULL, 0, 0 }
 };
 
@@ -590,30 +716,38 @@ void preset_lilo()
 
 void postload_lilo()
 {
+	expand_path(bx_options.lilo.kernel, sizeof(bx_options.lilo.kernel));
+	expand_path(bx_options.lilo.ramdisk, sizeof(bx_options.lilo.ramdisk));
 }
 
 void presave_lilo()
 {
+	compress_path(bx_options.lilo.kernel);
+	compress_path(bx_options.lilo.ramdisk);
 }
 
 /*************************************************************************/
 #define MIDI(x) bx_options.midi.x
 
 struct Config_Tag midi_conf[]={
-	{ "Enabled", Bool_Tag, &MIDI(enabled), 0, 0},
-	{ "Output", Path_Tag, &MIDI(output), sizeof(MIDI(output)), 0},
+	{ "Type", String_Tag, &MIDI(type), sizeof(MIDI(type)), 0},
+	{ "File", Path_Tag, &MIDI(file), sizeof(MIDI(file)), 0},
+	{ "Sequencer", Path_Tag, &MIDI(sequencer), sizeof(MIDI(sequencer)), 0},
 	{ NULL , Error_Tag, NULL, 0, 0 }
 };
 
 void preset_midi() {
-  MIDI(enabled)=false;
-  safe_strncpy(MIDI(output), "/tmp/aranym-midi.txt", sizeof(MIDI(output)));
+  safe_strncpy(MIDI(type), "none", sizeof(MIDI(type)));
+  safe_strncpy(MIDI(file), "", sizeof(MIDI(file)));
+  safe_strncpy(MIDI(sequencer), "/dev/sequencer", sizeof(MIDI(sequencer)));
 }
 
 void postload_midi() {
+	expand_path(bx_options.midi.file, sizeof(bx_options.midi.file));
 }
 
 void presave_midi() {
+	compress_path(bx_options.midi.file);
 }
 
 /*************************************************************************/
@@ -769,15 +903,6 @@ void presave_nfvdi() {
 }
 
 /*************************************************************************/
-#ifdef OS_darwin
-#define HOTKEY_OPENGUI		SDLK_PRINT	// F13
-#define	HOTKEY_FULLSCREEN	SDLK_NUMLOCK
-#define	HOTKEY_SCREENSHOT	SDLK_F15
-#else
-#define HOTKEY_OPENGUI		SDLK_PAUSE
-#define	HOTKEY_FULLSCREEN	SDLK_SCROLLOCK
-#define	HOTKEY_SCREENSHOT	SDLK_PRINT
-#endif
 
 #define HOTKEYS_STRING_SIZE		20
 static char hotkeys[10][HOTKEYS_STRING_SIZE];
@@ -792,7 +917,9 @@ struct Config_Tag hotkeys_conf[]={
 	{ NULL , Error_Tag, NULL, 0, 0 }
 };
 
-struct { char *string; SDL_keysym *keysym; } hotkeys_rel[]={
+typedef struct { char *string; SDL_keysym *keysym; } HOTKEYS_REL;
+
+HOTKEYS_REL hotkeys_rel[]={
 	{ hotkeys[0], &bx_options.hotkeys.setup },
 	{ hotkeys[1], &bx_options.hotkeys.quit },
 	{ hotkeys[2], &bx_options.hotkeys.reboot },
@@ -805,25 +932,42 @@ struct { char *string; SDL_keysym *keysym; } hotkeys_rel[]={
 void preset_hotkeys()
 {
 	// default values
-	bx_options.hotkeys.setup.sym = HOTKEY_OPENGUI;
+#ifdef OS_darwin
+	bx_options.hotkeys.setup.sym = SDLK_COMMA;
+	bx_options.hotkeys.setup.mod = KMOD_LMETA;
+	bx_options.hotkeys.quit.sym = SDLK_q;
+	bx_options.hotkeys.quit.mod = KMOD_LMETA;
+	bx_options.hotkeys.reboot.sym = SDLK_r;
+	bx_options.hotkeys.reboot.mod = KMOD_LMETA;
+	bx_options.hotkeys.ungrab.sym = SDLK_ESCAPE;
+	bx_options.hotkeys.ungrab.mod = KMOD_LMETA;
+	bx_options.hotkeys.debug.sym = SDLK_d;
+	bx_options.hotkeys.debug.mod = KMOD_LMETA;
+	bx_options.hotkeys.screenshot.sym = SDLK_s;
+	bx_options.hotkeys.screenshot.mod = KMOD_LMETA;
+	bx_options.hotkeys.fullscreen.sym = SDLK_f;
+	bx_options.hotkeys.fullscreen.mod = KMOD_LMETA;
+#else
+	bx_options.hotkeys.setup.sym = SDLK_PAUSE;
 	bx_options.hotkeys.setup.mod = KMOD_NONE;
-	bx_options.hotkeys.quit.sym = HOTKEY_OPENGUI;
+	bx_options.hotkeys.quit.sym = SDLK_PAUSE;
 	bx_options.hotkeys.quit.mod = KMOD_LSHIFT;
-	bx_options.hotkeys.reboot.sym = HOTKEY_OPENGUI;
+	bx_options.hotkeys.reboot.sym = SDLK_PAUSE;
 	bx_options.hotkeys.reboot.mod = KMOD_LCTRL;
 	bx_options.hotkeys.ungrab.sym = SDLK_ESCAPE;
 	bx_options.hotkeys.ungrab.mod = (SDLMod)(KMOD_LSHIFT | KMOD_LCTRL | KMOD_LALT);
-	bx_options.hotkeys.debug.sym = HOTKEY_OPENGUI;
+	bx_options.hotkeys.debug.sym = SDLK_PAUSE;
 	bx_options.hotkeys.debug.mod = KMOD_LALT;
-	bx_options.hotkeys.screenshot.sym = HOTKEY_SCREENSHOT;
+	bx_options.hotkeys.screenshot.sym = SDLK_PRINT;
 	bx_options.hotkeys.screenshot.mod = KMOD_NONE;
-	bx_options.hotkeys.fullscreen.sym = HOTKEY_FULLSCREEN;
+	bx_options.hotkeys.fullscreen.sym = SDLK_SCROLLOCK;
 	bx_options.hotkeys.fullscreen.mod = KMOD_NONE;
+#endif
 }
 
 void postload_hotkeys() {
 	// convert from string to pair of ints
-	for(uint i=0; i<sizeof(hotkeys_rel)/sizeof(hotkeys_rel[0]); i++) {
+	for(uint16 i=0; i<sizeof(hotkeys_rel)/sizeof(hotkeys_rel[0]); i++) {
 		int sym, mod;
 		if ( sscanf(hotkeys_rel[i].string, "%i:%i", &sym, &mod) != 2 ) continue;
 
@@ -834,11 +978,32 @@ void postload_hotkeys() {
 
 void presave_hotkeys() {
 	// convert from pair of ints to string
-	for(uint i=0; i<sizeof(hotkeys_rel)/sizeof(hotkeys_rel[0]); i++) {
+	for(uint16 i=0; i<sizeof(hotkeys_rel)/sizeof(hotkeys_rel[0]); i++) {
 		snprintf(hotkeys_rel[i].string, HOTKEYS_STRING_SIZE, "%d:%#x", hotkeys_rel[i].keysym->sym, hotkeys_rel[i].keysym->mod);
 	}
 }
 
+/*************************************************************************/
+struct Config_Tag ikbd_conf[]={
+	{ "WheelEiffel", Bool_Tag, &bx_options.ikbd.wheel_eiffel, 0, 0},
+	{ "AltGr", Bool_Tag, &bx_options.ikbd.altgr, 0, 0},
+	{ NULL , Error_Tag, NULL, 0, 0 }
+};
+
+void preset_ikbd()
+{
+	bx_options.ikbd.wheel_eiffel = false;
+	bx_options.ikbd.altgr = true;
+}
+
+void postload_ikbd()
+{
+}
+
+void presave_ikbd()
+{
+}
+
 /*************************************************************************/
 void usage (int status) {
   printf ("Usage: %s [OPTIONS]\n", program_name);
@@ -854,6 +1019,7 @@ Options:\n\
   -c, --config FILE          read different configuration file\n\
   -s, --save                 save configuration file\n\
   -S, --swap-ide             swap IDE drives\n\
+  -P <X,Y> or <center>       set window position\n\
   -h, --help                 display this help and exit\n\
   -V, --version              output version information and exit\n\
 ");
@@ -869,18 +1035,15 @@ Options:\n\
 #ifndef FixedSizeFastRAM
   printf("  -F, --fastram SIZE         FastRAM size (in MB)\n");
 #endif
-#if (defined(DEBUGGER) || defined(GDBSTUB))
+#ifdef DEBUGGER
   printf("  -D, --debug                start debugger\n");
-# ifdef GDBSTUB
-  printf("  -p, --port NUMBER          port number for GDB (default: 1234)\n");
-  printf("  -b, --base TEXT DATA BSS   base pointers\n");
-#endif
 #endif
   exit (status);
 }
 
 void preset_cfg() {
   preset_global();
+  preset_ikbd();
   preset_hotkeys();
   preset_ide();
   preset_disk();
@@ -903,6 +1066,7 @@ void preset_cfg() {
 
 void postload_cfg() {
   postload_global();
+  postload_ikbd();
   postload_hotkeys();
   postload_ide();
   postload_disk();
@@ -925,6 +1089,7 @@ void postload_cfg() {
 
 void presave_cfg() {
   presave_global();
+  presave_ikbd();
   presave_hotkeys();
   presave_ide();
   presave_disk();
@@ -989,12 +1154,8 @@ int process_cmdline(int argc, char **argv)
 #ifdef ENABLE_LILO
 							 "l"  /* boot lilo */
 #endif
-#if (defined(DEBUGGER) || defined(GDBSTUB))
+#ifdef DEBUGGER
 							 "D"  /* debugger */
-# ifdef GDBSTUB
-							 "p:" /* gdb port */
-							 "b:" /* base */
-# endif
 #endif
 #ifndef FixedSizeFastRAM
 							 "F:" /* FastRAM */
@@ -1013,6 +1174,7 @@ int process_cmdline(int argc, char **argv)
 #ifdef SDL_GUI
 							 "G"  /* GUI startup */
 #endif
+							 "P:" /* position of the window */
 							 "S"  /* swap IDE drives */
 							 "h"  /* help */
 							 "V"  /* version */,
@@ -1029,26 +1191,10 @@ int process_cmdline(int argc, char **argv)
 				startupGUI = true;
 				break;
 #endif
-#if (defined(DEBUGGER) || defined(GDBSTUB))
+#ifdef DEBUGGER
 			case 'D':
 				bx_options.startup.debugger = true;
 				break;
-# ifdef GDBSTUB
-			case 'p':
-				port_number = atoi(optarg); 
-				break;
-			case 'b':
-				if ((optind + 2 ) > argc)
-				{
-					fprintf(stderr, "Not enough parameters for  --base option");
-					usage(EXIT_FAILURE);
-				} else {
-					bx_options.gdbstub.text_base = atoi(optarg);
-					bx_options.gdbstub.data_base = atoi(argv[optind]);
-					bx_options.gdbstub.bss_base = atoi(argv[optind]);
-				}
-				break;
-# endif
 #endif
 
 			case 'e':
@@ -1087,6 +1233,10 @@ int process_cmdline(int argc, char **argv)
 				bx_options.video.boot_color_depth = atoi(optarg);
 				break;
 
+			case 'P':
+				safe_strncpy(bx_options.video.window_pos, optarg, sizeof(bx_options.video.window_pos));
+				break;
+
 #if HOSTFS_SUPPORT
 			case 'd':
 				if ( strlen(optarg) < 4 || optarg[1] != ':') {
@@ -1127,45 +1277,6 @@ int process_cmdline(int argc, char **argv)
 	return optind;
 }
 
-void postload_PathTag( const char *filename, struct Config_Tag *ptr ) {
-	char *path = (char *)ptr->buf;
-	if ( !strlen(path) )
-		return;
-
-	char home[2048];
-	const char *prefix = NULL;
-	size_t prefixLen = 0;
-
-	if ( path[0] == '~' ) {
-		// replace with the home folder path
-		prefix = Host::getHomeFolder(home, sizeof(home));
-		prefixLen = strlen( prefix );
-		path++;
-	} else if (path[0] == '*') {
-		prefix= Host::getDataFolder(home, sizeof(home));
-		prefixLen = strlen( prefix );
-		path++;
-	} else if ( path[0] != '/' && path[0] != '\\' && path[1] != ':' ) {
-		// find the last dirseparator
-		const char *slash = &filename[strlen(filename)];
-		while ( --slash >= filename &&
-			*slash != '/' &&
-			*slash != '\\' );
-		// get the path part (in front of the slash)
-		prefix = filename;
-		prefixLen = slash - prefix + 1;
-	}
-
-	if ( prefixLen > 0 ) {
-		if ( (size_t)ptr->buf_size >= prefixLen + strlen(path) ) {
-			memmove( (char*)ptr->buf + prefixLen, path, strlen(path)+1 );	
-			memmove( (char*)ptr->buf, prefix, prefixLen );
-		} else {
-			fprintf(stderr, "Error - config entry size is insufficient\n" );
-		}
-	}
-}
-
 // append a filename to a path
 char *addFilename(char *buffer, const char *file, unsigned int bufsize)
 {
@@ -1195,7 +1306,7 @@ char *getConfFilename(const char *file, char *buffer, unsigned int bufsize)
 	struct stat buf;
 	if (stat(buffer, &buf) == -1) {
 		D(bug("Creating config folder '%s'", buffer));
-		mkdir(buffer, 0755);
+		Host::makeDir(buffer);
 	}
 
 	return addFilename(buffer, file, bufsize);
@@ -1214,10 +1325,6 @@ static int process_config(FILE *f, const char *filename, struct Config_Tag *conf
 	if (status >= 0) {
 		if (verbose)
 			fprintf(f, "%s configuration: found %d valid directives.\n", title, status);
-		struct Config_Tag *ptr;
-		for ( ptr = conf; ptr->buf; ++ptr )
-			if ( ptr->type == Path_Tag )
-				postload_PathTag(filename, ptr);
 	} else {
 		fprintf(f, "Error while reading/processing the '%s' config file.\n", filename);
 	}
@@ -1240,8 +1347,9 @@ static bool decode_ini_file(FILE *f, const char *rcfile)
 	fprintf(f, "Using config file: '%s'\n", rcfile);
 
 	process_config(f, rcfile, global_conf, "[GLOBAL]", true);
-	process_config(f, rcfile, hotkeys_conf, "[HOTKEYS]", true);
 	process_config(f, rcfile, startup_conf, "[STARTUP]", true);
+	process_config(f, rcfile, ikbd_conf, "[IKBD]", true);
+	process_config(f, rcfile, hotkeys_conf, "[HOTKEYS]", true);
 	process_config(f, rcfile, jit_conf, "[JIT]", true);
 	process_config(f, rcfile, video_conf, "[VIDEO]", true);
 	process_config(f, rcfile, tos_conf, "[TOS]", true);
@@ -1290,8 +1398,9 @@ bool saveSettings(const char *fs)
 		fprintf(stderr, "Error while writing the '%s' config file.\n", fs);
 		return false;
 	}
-	update_config(fs, hotkeys_conf, "[HOTKEYS]");
 	update_config(fs, startup_conf, "[STARTUP]");
+	update_config(fs, ikbd_conf, "[IKBD]");
+	update_config(fs, hotkeys_conf, "[HOTKEYS]");
 	update_config(fs, jit_conf, "[JIT]");
 	update_config(fs, video_conf, "[VIDEO]");
 	update_config(fs, tos_conf, "[TOS]");
@@ -1340,12 +1449,12 @@ bool saveSettings(const char *fs)
 bool check_cfg()
 {
 #if REAL_ADDRESSING
-# if defined(__i386__) && defined(OS_linux)
+# if defined(CPU_i386) && defined(OS_linux)
 	if (bx_options.fastram > (128 - 16))
 	{
 		panicbug("Maximum Fast RAM size for real addressing on x86/Linux is 112 MB");
 		panicbug("If you need bigger Fast RAM, you must recompile ARAnyM");
-		panicbug("./configure --enable-addressing=direct");
+		panicbug("Use ./configure --enable-addressing=direct");
 		return false;
 	}
 # endif
@@ -1356,9 +1465,6 @@ bool check_cfg()
 bool decode_switches(FILE *f, int argc, char **argv)
 {
 	getConfFilename(ARANYMCONFIG, config_file, sizeof(config_file));
-	getDataFilename(TOS_FILENAME, bx_options.tos_path, sizeof(bx_options.tos_path));
-	getDataFilename(EMUTOS_FILENAME, bx_options.emutos_path, sizeof(bx_options.emutos_path));
-	getDataFilename(FREEMINT_FILENAME, bx_options.bootstrap_path, sizeof(bx_options.bootstrap_path));
 
 	early_cmdline_check(argc, argv);
 	preset_cfg();
