@@ -229,13 +229,13 @@ PRIVATE inline fpu_register FFPU make_single(uae_u32 value)
 		return (0.0);
 	
 	fpu_register result;
-	uae_u32 * p = (uae_u32 *)&result;
+	fpu_register_parts *p = (fpu_register_parts *)&result;
 
 	uae_u32 sign = (value & 0x80000000);
 	uae_u32 exp  = ((value & 0x7F800000) >> 23) + 1023 - 127;
 
-	p[FLO] = value << 29;
-	p[FHI] = sign | (exp << 20) | ((value & 0x007FFFFF) >> 3);
+	p->parts[FLO] = value << 29;
+	p->parts[FHI] = sign | (exp << 20) | ((value & 0x007FFFFF) >> 3);
 
 	fpu_debug(("make_single (%X) = %.04f\n",value,(double)result));
 	
@@ -259,10 +259,10 @@ PRIVATE inline uae_u32 FFPU extract_single(fpu_register const & src)
 		return 0;
 	
 	uae_u32 result;
-	uae_u32 *p = (uae_u32 *)&src;
+	fpu_register_parts const *p = (fpu_register_parts const *)&src;
 
-	uae_u32 sign = (p[FHI] & 0x80000000);
-	uae_u32 exp  = (p[FHI] & 0x7FF00000) >> 20;
+	uae_u32 sign = (p->parts[FHI] & 0x80000000);
+	uae_u32 exp  = (p->parts[FHI] & 0x7FF00000) >> 20;
 
 	if(exp + 127 < 1023) {
 		exp = 0;
@@ -272,7 +272,7 @@ PRIVATE inline uae_u32 FFPU extract_single(fpu_register const & src)
 		exp = exp + 127 - 1023;
 	}
 
-	result = sign | (exp << 23) | ((p[FHI] & 0x000FFFFF) << 3) | (p[FLO] >> 29);
+	result = sign | (exp << 23) | ((p->parts[FHI] & 0x000FFFFF) << 3) | (p->parts[FLO] >> 29);
 
 	fpu_debug(("extract_single (%.04f) = %X\n",(double)src,result));
 
@@ -285,17 +285,20 @@ PRIVATE inline fpu_register FFPU make_extended(uae_u32 wrd1, uae_u32 wrd2, uae_u
 {
 	// is it zero?
 	if ((wrd1 & 0x7fff0000) == 0 && wrd2 == 0 && wrd3 == 0)
-		return 0.0;
+		if (wrd1 & 0x80000000)
+			return -0.0;
+		else
+			return 0.0;
 
 	fpu_register result;
 #if USE_QUAD_DOUBLE
 	// is it NaN?
-	if ((wrd1 & 0x7fff0000) == 0x7fff0000 && wrd2 != 0 && wrd3 != 0) {
+	if ((wrd1 & 0x7fff0000) == 0x7fff0000 && ((wrd2 & 0x7fffffff) != 0 || wrd3 != 0)) {
 		make_nan(result);
 		return result;
 	}
 	// is it inf?
-	if ((wrd1 & 0x7ffff000) == 0x7fff0000 && wrd2 == 0 && wrd3 == 0) {
+	if ((wrd1 & 0x7ffff000) == 0x7fff0000 && (wrd2 & 0x7fffffff) == 0 && wrd3 == 0) {
 		if ((wrd1 & 0x80000000) == 0)
 			make_inf_positive(result);
 		else
@@ -365,17 +368,20 @@ PRIVATE inline void FFPU make_extended_no_normalize(
 {
 	// is it zero?
 	if ((wrd1 && 0x7fff0000) == 0 && wrd2 == 0 && wrd3 == 0) {
-		make_zero_positive(result);
+		if (wrd1 & 0x80000000)
+			make_zero_negative(result);
+		else
+			make_zero_positive(result);
 		return;
 	}
 	// is it NaN?
-	if ((wrd1 & 0x7fff0000) == 0x7fff0000 && wrd2 != 0 && wrd3 != 0) {
+	if ((wrd1 & 0x7fff0000) == 0x7fff0000 && ((wrd2 & 0x7fffffff) != 0 || wrd3 != 0)) {
 		make_nan(result);
 		return;
 	}
 #if USE_QUAD_DOUBLE
 	// is it inf?
-	if ((wrd1 & 0x7ffff000) == 0x7fff0000 && wrd2 == 0 && wrd3 == 0) {
+	if ((wrd1 & 0x7ffff000) == 0x7fff0000 && (wrd2 & 0x7fffffff) == 0 && wrd3 == 0) {
 		if ((wrd1 & 0x80000000) == 0)
 			make_inf_positive(result);
 		else
@@ -431,15 +437,15 @@ PRIVATE inline void FFPU extract_extended(fpu_register const & src,
 	*wrd2 = 0x80000000 | (srp->ieee.mantissa0 << 15) | ((srp->ieee.mantissa1 & 0xfffe0000) >> 17);
 	*wrd3 = (srp->ieee.mantissa1 << 15) | ((srp->ieee.mantissa2 & 0xfffe0000) >> 17);
 #elif USE_LONG_DOUBLE
-	uae_u32 *p = (uae_u32 *)&src;
+	fpu_register_parts const *p = (fpu_register_parts const *)&src;
 #ifdef WORDS_BIGENDIAN
-	*wrd1 = p[0];
-	*wrd2 = p[1];
-	*wrd3 = p[2];
+	*wrd1 = p->parts[0];
+	*wrd2 = p->parts[1];
+	*wrd3 = p->parts[2];
 #else
-	*wrd3 = p[0];
-	*wrd2 = p[1];
-	*wrd1 = ( (uae_u32)*((uae_u16 *)&p[2]) ) << 16;
+	*wrd3 = p->parts[0];
+	*wrd2 = p->parts[1];
+	*wrd1 = (p->parts[2] & 0xffff) << 16;
 #endif
 #else
 	fp_declare_init_shape(srp, src, double);
@@ -551,7 +557,7 @@ PRIVATE inline void FFPU extract_packed(fpu_register const & src, uae_u32 * wrd1
 	char *cp;
 	char str[100];
 
-	sprintf(str, "%.16Le", src);
+	sprintf(str, "%.16Le", (long double)src);
 
 	fpu_debug(("extract_packed(%.04f,%s)\n",(double)src,str));
 
@@ -690,8 +696,8 @@ PRIVATE inline int FFPU get_fp_value (uae_u32 opcode, uae_u16 extra, fpu_registe
 	fpu_debug(("get_fp_value m68k_getpc()=%X\n",m68k_getpc()));
 	fpu_debug(("get_fp_value ad=%X\n",ad));
 	fpu_debug(("get_fp_value get_long (ad)=%X\n",get_long (ad)));
-	dump_first_bytes( get_real_address(ad, 0, 0)-64, 64 );
-	dump_first_bytes( get_real_address(ad, 0, 0), 64 );
+	//dump_first_bytes( get_real_address(ad, 0, 0)-64, 64 );
+	//dump_first_bytes( get_real_address(ad, 0, 0), 64 );
 
 	switch (size) {
 	case 0:
@@ -2093,7 +2099,17 @@ void FFPU fpuop_arithmetic(uae_u32 opcode, uae_u32 extra)
 		case 0x38:		/* FCMP */
 			fpu_debug(("FCMP %.04f\n",(double)src));
 			set_fpsr(0);
-			make_fpsr(FPU registers[reg] - src);
+			if (isinf(FPU registers[reg]))
+			{
+				if (isinf(src) && isneg(FPU registers[reg]) == isneg (src))
+					make_fpsr(0);
+				else
+					make_fpsr(FPU registers[reg]);
+			}
+			else if (isinf(src))
+				make_fpsr(-src);
+      else
+				make_fpsr(FPU registers[reg] - src);
 			break;
 		case 0x3a:		/* FTST */
 			fpu_debug(("FTST %.04f\n",(double)src));
