@@ -311,7 +311,7 @@ union fpu_extended_shape {
 		unsigned int exponent:15;
 		unsigned int quiet_nan:1;
 		unsigned int mantissa0:15;
-		unsigned int mantissa1:30;
+		unsigned int mantissa1:32;
 		unsigned int mantissa2:32;
 		unsigned int mantissa3:32;
 #else
@@ -449,7 +449,7 @@ PRIVATE inline bool FFPU fp_do_isinf(fpu_register const & r)
 	/* This additional ^ 0x80000000 is necessary because in Intel's
 	   internal representation of the implicit one is explicit.
 	   NOTE: anyway, this is equivalent to & 0x7fffffff in that case.  */
-#ifdef __i386__
+#ifdef CPU_i386
 	lx |= (hx ^ 0x80000000) | ((se & 0x7fff) ^ 0x7fff);
 #else
 	lx |= (hx & 0x7fffffff) | ((se & 0x7fff) ^ 0x7fff);
@@ -833,6 +833,13 @@ PRIVATE fpu_extended fp_do_exp(fpu_extended x);
 PRIVATE inline fpu_extended fp_do_exp(fpu_extended x)
 {
 	fpu_extended value, exponent;
+	if (isinf(x))
+	{
+		if(isneg(x))
+			return 0.;
+		else
+			return x;
+	}
 	__asm__ __volatile__("fldl2e                    # e^x = 2^(x * log2(e))\n\t"
 				 "fmul      %%st(1)         # x * log2(e)\n\t"
 				 "fst       %%st(1)\n\t"
@@ -872,6 +879,7 @@ PRIVATE inline fpu_extended fp_do_sqrt(fpu_extended x)
 	return value;
 }
 
+#ifndef ACCURATE_SIN_COS_TAN
 #undef fp_sin
 #define fp_sin fp_do_sin
 
@@ -897,10 +905,11 @@ PRIVATE inline fpu_extended fp_do_cos(fpu_extended x)
 
 PRIVATE inline fpu_extended fp_do_tan(fpu_extended x)
 {
-	fpu_extended value, value2;
+	fpu_extended value;
 	__asm__ __volatile__("fptan" : "=t" (value), "=u" (value) : "0" (x));
 	return value;
 }
+#endif /* ACCURATE_SIN_COS_TAN */
 
 #undef fp_expm1
 #define fp_expm1 fp_do_expm1
@@ -909,6 +918,13 @@ PRIVATE inline fpu_extended fp_do_tan(fpu_extended x)
 PRIVATE inline fpu_extended fp_do_expm1(fpu_extended x)
 {
 	fpu_extended value, exponent, temp;
+	if (isinf(x))
+	{
+		if(isneg(x))
+			return -1.;
+		else
+			return x;
+	}
 	__asm__ __volatile__("fldl2e                    # e^x - 1 = 2^(x * log2(e)) - 1\n\t"
 				 "fmul      %%st(1)         # x * log2(e)\n\t"
 				 "fst       %%st(1)\n\t"
@@ -930,11 +946,11 @@ PRIVATE inline fpu_extended fp_do_sgn1(fpu_extended x)
 {
 #if USE_LONG_DOUBLE || USE_QUAD_DOUBLE
 	fp_declare_init_shape(sxp, x, extended);
-	sxp->ieee_nan.exponent	= FP_EXTENDED_EXP_MAX;
+	sxp->ieee_nan.exponent	= FP_EXTENDED_EXP_MAX>>1;
 	sxp->ieee_nan.one		= 1;
 #else
 	fp_declare_init_shape(sxp, x, double);
-	sxp->ieee_nan.exponent  = FP_DOUBLE_EXP_MAX;
+	sxp->ieee_nan.exponent  = FP_DOUBLE_EXP_MAX>>1;
 #endif
 	sxp->ieee_nan.quiet_nan	= 0;
 	sxp->ieee_nan.mantissa0	= 0;
@@ -951,10 +967,9 @@ PRIVATE fpu_extended fp_do_sinh(fpu_extended x);
 #else
 PRIVATE inline fpu_extended fp_do_sinh(fpu_extended x)
 {
-	fpu_extended ex = fp_exp(x);
-	return 0.5 * (ex - 1.0 / ex);
-/*	fpu_extended exm1 = fp_expm1(fp_fabs(x));
-	return 0.5 * (exm1 / (exm1 + 1.0) + exm1) * fp_sgn1(x); */
+	if (isinf(x)) return x;
+	fpu_extended exm1 = fp_expm1(fp_fabs(x));
+	return 0.5 * (exm1 / (exm1 + 1.0) + exm1) * fp_sgn1(x);
 }
 #endif
 
@@ -981,10 +996,8 @@ PRIVATE fpu_extended fp_do_tanh(fpu_extended x);
 #else
 PRIVATE inline fpu_extended fp_do_tanh(fpu_extended x)
 {
-	fpu_extended ex = fp_exp(2.0 * x);
-	return (ex - 1.0) / (ex + 1.0);
-/*	fpu_extended exm1 = fp_expm1(-fp_fabs(x + x));
-	return exm1 / (exm1 + 2.0) * fp_sgn1(-x); */
+	fpu_extended exm1 = fp_expm1(-fp_fabs(x + x));
+	return exm1 / (exm1 + 2.0) * fp_sgn1(-x);
 }
 #endif
 
