@@ -31,6 +31,7 @@
   */
 
 #include "sysdeps.h"
+#include <cassert>
 
 #include "cpu_emulation.h"
 #include "main.h"
@@ -52,6 +53,8 @@
 
 #define SANITY_CHECK_ATC 1
 
+struct fixup fixup = {0, 0, 0};
+
 int quit_program = 0;
 
 // For instruction $7139
@@ -91,11 +94,20 @@ int movem_next[256];
 cpuop_func *cpufunctbl[65536];
 
 #if FLIGHT_RECORDER
+
+// feel free to edit the following defines to customize the dump
+#define FRLOG_HOTKEY	1	/* 1 = dump only when hotkey is held down */
+#define FRLOG_ALL	1	/* 1 = dump continuously to ever growing log */
+#define FRLOG_IRQ	0	/* 1 = dump also CPU in interrupts */
+#define FRLOG_REGS	0	/* 1 = dump also all data/address registers */
+#define FRLOG_SIZE	8192	/* this many instructions in single dump */
+
 struct rec_step {
 	uae_u32 d[8];
 	uae_u32 a[8];
 	uae_u32 pc;
 	uae_u16 sr;
+	uae_u32 usp;
 	uae_u32 msp;
 	uae_u32 isp;
 	uae_u16 instr;
@@ -103,7 +115,11 @@ struct rec_step {
 
 bool cpu_flight_recorder_active = false;
 
-const int LOG_SIZE = 8192;
+#if FRLOG_ALL
+const int LOG_SIZE = 10;
+#else
+const int LOG_SIZE = FRLOG_SIZE;
+#endif
 static rec_step log[LOG_SIZE];
 static int log_ptr = -1; // First time initialization
 
@@ -113,10 +129,34 @@ static const char *log_filename(void)
 	return name ? name : "log.68k";
 }
 
-static void dump_log();
-void m68k_record_step(uaecptr pc)
+static void dump_log()
+{
+#if FRLOG_ALL
+	FILE *f = fopen(log_filename(), "a");
+#else
+	FILE *f = fopen(log_filename(), "w");
+#endif
+	if (f == NULL)
+		return;
+	for (int i = 0; i < LOG_SIZE; i++) {
+		int j = (i + log_ptr) % LOG_SIZE;
+		fprintf(f, "pc %08x  instr %04x  sr %04x  usp %08x  msp %08x  isp %08x\n", log[j].pc, log[j].instr, log[j].sr, log[j].usp, log[j].msp, log[j].isp);
+	// adding a simple opcode -> assembler conversion table would help
+#if FRLOG_REGS
+		fprintf(f, "d0 %08x d1 %08x d2 %08x d3 %08x\n", log[j].d[0], log[j].d[1], log[j].d[2], log[j].d[3]);
+		fprintf(f, "d4 %08x d5 %08x d6 %08x d7 %08x\n", log[j].d[4], log[j].d[5], log[j].d[6], log[j].d[7]);
+		fprintf(f, "a0 %08x a1 %08x a2 %08x a3 %08x\n", log[j].a[0], log[j].a[1], log[j].a[2], log[j].a[3]);
+		fprintf(f, "a4 %08x a5 %08x a6 %08x a7 %08x\n", log[j].a[4], log[j].a[5], log[j].a[6], log[j].a[7]);
+#endif
+	}
+	fclose(f);
+}
+
+void m68k_record_step(uaecptr pc, int opcode)
 {
 	static bool last_state = false;
+
+#if FRLOG_HOTKEY
 	if (! cpu_flight_recorder_active) {
 		if (last_state) {
 			// dump log out
@@ -127,61 +167,42 @@ void m68k_record_step(uaecptr pc)
 		}
 		return;
 	}
+#endif
 
 	if (! last_state) {
 		// reset old log
-    	log_ptr = 0;
-    	memset(log, 0, sizeof(log));
+		log_ptr = 0;
+		memset(log, 0, sizeof(log));
 		// remember last state
 		last_state = true;
 	}
 
+#if FRLOG_REGS
 	for (int i = 0; i < 8; i++) {
 		log[log_ptr].d[i] = m68k_dreg(regs, i);
 		log[log_ptr].a[i] = m68k_areg(regs, i);
 	}
+#endif
 	log[log_ptr].pc = pc;
+
 	MakeSR();
+#if ! FRLOG_IRQ
+	// is CPU in interrupt handler? Quit if should not be logged.
+	if (regs.s && !regs.m) return;
+#endif
 	log[log_ptr].sr = regs.sr;
+	log[log_ptr].usp = regs.usp;
 	log[log_ptr].msp = regs.msp;
 	log[log_ptr].isp = regs.isp;
-	TRY(prb) {
-		log[log_ptr].instr = GET_OPCODE;
-	} CATCH(prb) {
-		log[log_ptr].instr = M68K_EMUL_OP_MAX;
-	}
-	log_ptr = (log_ptr + 1) % LOG_SIZE;
-}
+	log[log_ptr].instr = opcode;
 
-static void dump_log(void)
-{
-	FILE *f = fopen(log_filename(), "w");
-	if (f == NULL)
-		return;
-	for (int i = 0; i < LOG_SIZE; i++) {
-		int j = (i + log_ptr) % LOG_SIZE;
-		fprintf(f, "pc %08x sr %04x msp %08x isp %08x\n", log[j].pc, log[j].sr, log[j].msp, log[j].isp);
-		fprintf(f, "d0 %08x d1 %08x d2 %08x d3 %08x\n", log[j].d[0], log[j].d[1], log[j].d[2], log[j].d[3]);
-		fprintf(f, "d4 %08x d5 %08x d6 %08x d7 %08x\n", log[j].d[4], log[j].d[5], log[j].d[6], log[j].d[7]);
-		fprintf(f, "a0 %08x a1 %08x a2 %08x a3 %08x\n", log[j].a[0], log[j].a[1], log[j].a[2], log[j].a[3]);
-		fprintf(f, "a4 %08x a5 %08x a6 %08x a7 %08x\n", log[j].a[4], log[j].a[5], log[j].a[6], log[j].a[7]);
-		fprintf(f, "instr: %04x\n", log[j].instr);
-#if ENABLE_MON
-		disass_68k(f, log[j].pc);
+	log_ptr = (log_ptr + 1) % LOG_SIZE;
+#if FRLOG_ALL
+	if (log_ptr == 0) dump_log();
 #endif
-	}
-	fclose(f);
 }
 #endif /* FLIGHT_RECORDER */
 
-#if ENABLE_MON
-static void dump_regs(void)
-{
-	m68k_dumpstate(NULL);
-}
-#endif
-
-
 int broken_in;
 
 static inline unsigned int cft_map (unsigned int f)
@@ -279,25 +300,15 @@ int lastint_no;
 #ifdef FULLMMU
 static inline uae_u8 get_ibyte_1(uae_u32 o)
 {
-    uaecptr addr = m68k_getpc() + o + 1;
-    return phys_get_byte(mmu_translate(addr, FC_INST, 0, sz_byte, 0));
+    return get_ibyte(o);
 }
 static inline uae_u16 get_iword_1(uae_u32 o)
 {
-    uaecptr addr = m68k_getpc() + o;
-    return phys_get_word(mmu_translate(addr, FC_INST, 0, sz_word, 0));
+    return get_iword(o);
 }
 static inline uae_u32 get_ilong_1(uae_u32 o)
 {
-    uaecptr addr = m68k_getpc() + o;
-    if (is_unaligned(addr, 4)) {
-	uae_u32 result;
-	result = phys_get_word(mmu_translate(addr, FC_INST, 0, sz_word, 0));
-	result <<= 16;
-	result |= phys_get_word(mmu_translate(addr + 2, FC_INST, 0, sz_word, 0));
-	return result;
-    }
-    return phys_get_long(mmu_translate(addr, FC_INST, 0, sz_long, 0));
+    return get_ilong(o);
 }
 #else
 # define get_ibyte_1(o) get_byte(m68k_getpc() + (o) + 1)
@@ -607,6 +618,84 @@ static int verify_ea (int reg, amodes mode, wordsizes size, uae_u32 *val)
 }
 #endif
 
+/*
+ * extract bitfield data from memory and return it in the MSBs
+ * bdata caches the unmodified data for put_bitfield()
+ */
+uae_u32 get_bitfield(uae_u32 src, uae_u32 bdata[2], uae_s32 offset, int width)
+{
+	uae_u32 tmp, res, mask;
+
+	offset &= 7;
+	mask = 0xffffffffu << (32 - width);
+	switch ((offset + width + 7) >> 3) {
+	case 1:
+		tmp = get_byte(src);
+		res = tmp << (24 + offset);
+		bdata[0] = tmp & ~(mask >> (24 + offset));
+		break;
+	case 2:
+		tmp = get_word(src);
+		res = tmp << (16 + offset);
+		bdata[0] = tmp & ~(mask >> (16 + offset));
+		break;
+	case 3:
+		tmp = get_word(src);
+		res = tmp << (16 + offset);
+		bdata[0] = tmp & ~(mask >> (16 + offset));
+		tmp = get_byte(src + 2);
+		res |= tmp << (8 + offset);
+		bdata[1] = tmp & ~(mask >> (8 + offset));
+		break;
+	case 4:
+		tmp = get_long(src);
+		res = tmp << offset;
+		bdata[0] = tmp & ~(mask >> offset);
+		break;
+	case 5:
+		tmp = get_long(src);
+		res = tmp << offset;
+		bdata[0] = tmp & ~(mask >> offset);
+		tmp = get_byte(src + 4);
+		res |= tmp >> (8 - offset);
+		bdata[1] = tmp & ~(mask << (8 - offset));
+		break;
+	default:
+		/* Panic? */
+		res = 0;
+		break;
+	}
+	return res;
+}
+
+/*
+ * write bitfield data (in the LSBs) back to memory, upper bits
+ * must be cleared already.
+ */
+void put_bitfield(uae_u32 dst, uae_u32 bdata[2], uae_u32 val, uae_s32 offset, int width)
+{
+	offset = (offset & 7) + width;
+	switch ((offset + 7) >> 3) {
+	case 1:
+		put_byte(dst, bdata[0] | (val << (8 - offset)));
+		break;
+	case 2:
+		put_word(dst, bdata[0] | (val << (16 - offset)));
+		break;
+	case 3:
+		put_word(dst, bdata[0] | (val >> (offset - 16)));
+		put_byte(dst + 2, bdata[1] | (val << (24 - offset)));
+		break;
+	case 4:
+		put_long(dst, bdata[0] | (val << (32 - offset)));
+		break;
+	case 5:
+		put_long(dst, bdata[0] | (val >> (offset - 32)));
+		put_byte(dst + 4, bdata[1] | (val << (40 - offset)));
+		break;
+	}
+}
+
 uae_u32 get_disp_ea_020 (uae_u32 base, uae_u32 dp)
 {
     int reg = (dp >> 12) & 15;
@@ -686,6 +775,7 @@ void MakeFromSR (void)
     regs.t1 = (regs.sr >> 15) & 1;
     regs.t0 = (regs.sr >> 14) & 1;
     regs.s = (regs.sr >> 13) & 1;
+    mmu_set_super(regs.s);
     regs.m = (regs.sr >> 12) & 1;
     regs.intmask = (regs.sr >> 8) & 7;
     SET_XFLG ((regs.sr >> 4) & 1);
@@ -768,10 +858,18 @@ void Exception(int nr, uaecptr oldpc)
 {
     uae_u32 currpc = m68k_getpc ();
     MakeSR();
+
+    if (fixup.flag)
+    {
+        m68k_areg(regs, fixup.reg) = fixup.value;
+        fixup.flag = 0;
+    }
+
     if (!regs.s) {
 	regs.usp = m68k_areg(regs, 7);
 	m68k_areg(regs, 7) = regs.m ? regs.msp : regs.isp;
 	regs.s = 1;
+	mmu_set_super(1);
     }
 
     if (nr == 2) {
@@ -1201,10 +1299,6 @@ static int m68k_execute_depth = 0;
 
 void m68k_reset (void)
 {
-    m68k_areg (regs, 7) = phys_get_long(0x00000000);
-    m68k_setpc (phys_get_long(0x00000004));
-    mmu_set_tc(regs.tc & ~0x8000); /* disable mmu */
-    fill_prefetch_0 ();
     regs.s = 1;
     regs.m = 0;
     regs.stopped = 0;
@@ -1218,17 +1312,21 @@ void m68k_reset (void)
     SPCFLAGS_INIT( 0 );
     regs.intmask = 7;
     regs.vbr = regs.sfc = regs.dfc = 0;
+
+    // need to ensure the following order of initialization is correct
+    // (it is definitely better than what it was before this commit
+    //  since it was reading from 0x00000000 in User mode and with active MMU)
+    mmu_set_tc(regs.tc & ~0x8000); /* disable mmu */
+    m68k_areg (regs, 7) = phys_get_long(0x00000000);
+    m68k_setpc (phys_get_long(0x00000004));
+    fill_prefetch_0 ();
+
     /* gb-- moved into {fpp,fpu_x86}.cpp::fpu_init()
     regs.fpcr = regs.fpsr = regs.fpiar = 0; */
     fpu_reset();
     // MMU
-    mmu_set_root_pointer(0x806, 0);	// regs.urp = 0;
-    mmu_set_root_pointer(0x807, 0);	// regs.srp = 0;
-    mmu_set_ttr(TTR_D0, 0);
-    mmu_set_ttr(TTR_D1, 0);
-    mmu_set_ttr(TTR_I0, 0);
-    mmu_set_ttr(TTR_I1, 0);
-    mmu_set_mmusr(0);
+    mmu_reset();
+    mmu_set_super(1);
     // Cache
     regs.cacr = 0;
     regs.caar = 0;
@@ -1236,18 +1334,6 @@ void m68k_reset (void)
 	log_ptr = 0;
 	memset(log, 0, sizeof(log));
 #endif
-
-#if ENABLE_MON
-	static bool first_time = true;
-	if (first_time) {
-		first_time = false;
-		mon_add_command("regs", dump_regs, "regs                    Dump m68k emulator registers\n");
-#if FLIGHT_RECORDER
-		// Install "log" command in mon
-		mon_add_command("log", dump_log, "log                      Dump m68k emulation log\n");
-#endif
-	}
-#endif
 }
 
 void m68k_emulop_return(void)
@@ -1378,23 +1464,6 @@ void REGPARAM2 op_illg (uae_u32 opcode)
 #endif
 }
 
-#ifndef FULLMMU
-void mmu_op(uae_u32 opcode, uae_u16 /*extra*/)
-{
-    if ((opcode & 0xFF8) == 0x0500) { /* PFLUSHN instruction (An) */
-	flush_internals();
-    } else if ((opcode & 0xFF8) == 0x0508) { /* PFLUSH instruction (An) */
-	flush_internals();
-    } else if ((opcode & 0xFF8) == 0x0510) { /* PFLUSHAN instruction */
-	flush_internals();
-    } else if ((opcode & 0xFF8) == 0x0518) { /* PFLUSHA instruction */
-	flush_internals();
-    } else if ((opcode & 0xFF8) == 0x548) { /* PTESTW instruction */
-    } else if ((opcode & 0xFF8) == 0x568) { /* PTESTR instruction */
-    } else op_illg(opcode);
-}
-#endif
-
 static uaecptr last_trace_ad = 0;
 
 static void do_trace (void)
@@ -1568,7 +1637,7 @@ void m68k_do_execute (void)
 #endif
 	opcode = GET_OPCODE;
 #if FLIGHT_RECORDER
-	                m68k_record_step(m68k_getpc());
+	m68k_record_step(m68k_getpc(), opcode);
 #endif
 	(*cpufunctbl[opcode])(opcode);
 	cpu_check_ticks();
@@ -1926,7 +1995,7 @@ void m68k_dumpstate (uaecptr *nextpc)
     printf ("T=%d%d S=%d M=%d X=%d N=%d Z=%d V=%d C=%d IMASK=%d TCE=%d TCP=%d\n",
 	    regs.t1, regs.t0, regs.s, regs.m,
 	    (int)GET_XFLG, (int)GET_NFLG, (int)GET_ZFLG, (int)GET_VFLG, (int)GET_CFLG, regs.intmask,
-	    regs.mmu_enabled, regs.mmu_pagesize);
+	    regs.mmu_enabled, regs.mmu_pagesize_8k);
     printf ("CACR=%08lx CAAR=%08lx  URP=%08lx  SRP=%08lx\n",
             (unsigned long)regs.cacr,
 	    (unsigned long)regs.caar,
