@@ -125,7 +125,7 @@ long VIDEL::getVideoramAddress()
 int VIDEL::getScreenBpp()
 {
 	int f_shift = handleReadW(HW + 0x66);
-	int st_shift = handleReadW(HW + 0x60);
+	int st_shift = handleRead(HW + 0x60);
 	/* to get bpp, we must examine f_shift and st_shift.
 	 * f_shift is valid if any of bits no. 10, 8 or 4
 	 * is set. Priority in f_shift is: 10 ">" 8 ">" 4, i.e.
@@ -142,9 +142,9 @@ int VIDEL::getScreenBpp()
 		bits_per_pixel = 8;
 	else if (st_shift == 0)
 		bits_per_pixel = 4;
-	else if (st_shift == 0x100)
+	else if (st_shift == 0x01)
 		bits_per_pixel = 2;
-	else /* if (st_shift == 0x200) */
+	else /* if (st_shift == 0x02) */
 		bits_per_pixel = 1;
 
 	// D(bug("Videl works in %d bpp, f_shift=%04x, st_shift=%d", bits_per_pixel, f_shift, st_shift));
@@ -185,7 +185,7 @@ void VIDEL::updateColors()
 	// Test the ST compatible set or not.
 	bool stCompatibleColorPalette = false;
 
-	int st_shift = handleReadW(HW + 0x60);
+	int st_shift = handleRead(HW + 0x60);
 	if (st_shift == 0) {		   // bpp == 4
 		int hreg = handleReadW(HW + 0x82); // Too lame!
 		if (hreg == 0x10 | hreg == 0x17 | hreg == 0x3e)	  // Better way how to make out ST LOW mode wanted
@@ -193,12 +193,13 @@ void VIDEL::updateColors()
 
 		D(bug("ColorUpdate %x", hreg));
 	}
-	else if (st_shift == 0x100)	   // bpp == 2
+	else if (st_shift == 0x01)	   // bpp == 2
 		stCompatibleColorPalette = true;
 	else						   // bpp == 1	// if (st_shift == 0x200)
 		stCompatibleColorPalette = true;
 
 	// map the colortable into the correct pixel format
+	int r,g,b;
 
 #define F_COLORS(i) handleRead(VIDEL_COLOR_REGS_BEGIN + (i))
 #define STE_COLORS(i)	handleRead(0xff8240 + (i))
@@ -206,22 +207,28 @@ void VIDEL::updateColors()
 	if (!stCompatibleColorPalette) {
 		for (int i = 0; i < 256; i++) {
 			int offset = i << 2;
-			hostScreen.setPaletteColor( i,
-										F_COLORS(offset),
-										F_COLORS(offset + 1),
-										F_COLORS(offset + 3) );
+			r = F_COLORS(offset) & 0xfc;
+			r |= r>>6;
+			g = F_COLORS(offset + 1) & 0xfc;
+			g |= g>>6;
+			b = F_COLORS(offset + 3) & 0xfc;
+			b |= b>>6;
+			hostScreen.setPaletteColor(i, r,g,b);
 		}
 		hostScreen.updatePalette( 256 );
 	} else {
 		for (int i = 0; i < 16; i++) {
 			int offset = i << 1;
-
-			// fprintf(stderr,"HS: setColor: %03d,%6x - %x,%x,%x\n", index, sdl_colors[index], red, green, blue );
-
-			hostScreen.setPaletteColor( i,
-										(STE_COLORS(offset) << 5) | ((STE_COLORS(offset) << 1 ) & 0x8),
-										((STE_COLORS(offset + 1) << 1) & 0xE0) | ((STE_COLORS(offset + 1) >> 3) & 0x8),
-										((STE_COLORS(offset + 1) << 5) & 0xE0) | ((STE_COLORS(offset + 1) << 1) & 0x8));
+			r = STE_COLORS(offset) & 0x0f;
+			r = ((r & 7)<<1)|(r>>3);
+			r |= r<<4;
+			g = (STE_COLORS(offset + 1)>>4) & 0x0f;
+			g = ((g & 7)<<1)|(g>>3);
+			g |= g<<4;
+			b = STE_COLORS(offset + 1) & 0x0f;
+			b = ((b & 7)<<1)|(b>>3);
+			b |= b<<4;
+			hostScreen.setPaletteColor(i, r,g,b);
 		}
 		hostScreen.updatePalette( 16 );
 	}
